/*
 * Author: 黎永顺
 * name: 薪资核算-列表
 * Description:
 * Date: 2023/9/14
 */
import React, { FunctionComponent, useContext, useEffect, useRef, useState } from "react";
import { Button, Form, Input, InputNumber, Table, Typography } from "antd";
import type { FormInstance } from "antd/es/form";
import { LockOutlined } from "@ant-design/icons";
import CustomTableTitle from "@/pages/calcTable/customTableTitle";
import CalcExplainFooter from "@/pages/calcTable/calcExplainFooter";
import CaclFixedTotal from "./calcFixedTotal";
import type { ColumnType } from "antd/lib/table";
import type { PaginationData } from "rc-pagination";
import { exceptStr, paginationFun, toDecimal_n } from "@/utils/common";
import { IPage } from "@/common/types";
import styles from "@/pages/atdTable/components/index.less";
import cs from "classnames";

interface OwnProps {
}

interface EditableRowProps {
}

interface EditableCellProps {
  title?: React.ReactNode;
  editable?: boolean;
  children?: React.ReactNode;
  dataIndex?: any;
  record?: any;
  pattern?: number;
  rowIndex?: number;
  handleSave?: (type: string, record: any) => void;
}

type Props = OwnProps;
type fixedProps = boolean | "top" | "bottom";
const { Text } = Typography;

const EditableContext = React.createContext<FormInstance<any> | null>(null);
const Index: FunctionComponent<Props> = (props) => {
  const [columns, setColumns] = useState<ColumnType<any>[]>([]);
  const [dataSource, setDataSource] = useState<any[]>([]);
  const [selectedRowKeys, setSelectedRowKeys] = useState<React.Key[]>([]);
  const [pageInfo, setPageInfo] = useState<Partial<PaginationData>>({});
  const [i18n, setI18n] = useState<any>({});
  const [showTotalCell, setShowTotalCell] = useState<boolean>(false);
  const [isDetailTable, setIsDetailTable] = useState<boolean>(false);
  const [sumRowlistUrl, setSumRowlistUrl] = useState<string>("");
  const [tableScrollHeight, setTableScrollHeight] = useState<Number>(0);
  const [payload, setPayload] = useState<string>("");
  const [fixed, setFixed] = useState<fixedProps>(true);
  const [sumRow, setSumRow] = useState<Partial<{}>>({});//总计行数据
  const [form] = Form.useForm();
  const editItemList = useRef<any[]>([]);

  useEffect(() => {
    window.parent.postMessage({ type: "init" }, "*");
    window.addEventListener("message", receiveMessageFromIndex, false);
    return () => {
      window.removeEventListener("message", receiveMessageFromIndex, false);
    };
  }, []);
  const receiveMessageFromIndex = (event: any) => {
    const data: any = exceptStr(event.data);
    if (!_.isEmpty(data)) {
      const {
        columns, dataSource, pageInfo, selectedRowKeys, i18n: i18nRes = {},
        showTotalCell = false, sumRowlistUrl = "", payload = {}, calcDetail,
        fixed = true, tableScrollHeight, sumRow
      } = data;
      setSumRowlistUrl(sumRowlistUrl);
      setShowTotalCell(showTotalCell);
      setIsDetailTable(calcDetail);
      setI18n(i18nRes);
      setPayload(payload);
      setFixed(fixed);
      setSumRow(sumRow);
      setPageInfo(pageInfo);
      setDataSource(dataSource);
      setSelectedRowKeys(selectedRowKeys);
      editItemList.current = [];
      setTableScrollHeight(tableScrollHeight);
      setColumns([...convertColumns(_.map(columns, o => ({ ...o, i18n: i18nRes })), calcDetail), {
        title: i18nRes["操作"], dataIndex: "operate", fixed: "right", width: 120,
        render: (__, record) => (<Button type="link" onClick={() => handleEdit(record?.id)}>{i18nRes["编辑"]}</Button>)
      }]);
    }
  };
  const convertColumns: any = (cols: any[], editable: boolean) => {
    return _.map(cols, item => {
      if (_.isNaN(parseInt(item.dataIndex))) {
        return { ...item, editable: false };
      } else {
        return {
          ...item, title: <CustomTableTitle {...item} onHandleFormulatd={handleFormulaTd}/>,
          children: convertColumns(_.map(item.children, o => ({ ...o, i18n: item.i18n })), editable),
          className: styles["td_odd"], i18n: item.i18n, editable: !editable,
          onCell: (record: any, rowIndex: number) => ({
            record,
            rowIndex,
            dataIndex: item.dataIndex,
            title: item.title,
            editable: !editable,
            pattern: item.pattern,
            handleSave
          }),
          render: (text: string) => (
            <span className={styles.contentSpan}>
              <span title={text} className={styles.contentTitle}>{text}</span>
              {
                item.lockStatus === "LOCK" ? <LockOutlined title={i18n["锁定的项目值"]}/> : null
              }
            </span>
          )
        };
      }
    });
  };
  const handleSave = (salaryItemId: string, row: any) => {
    setDataSource(prevState => {
      const newData = [...prevState];
      const index = newData.findIndex(item => row.id === item.id);
      const item = newData[index];
      if (_.map(editItemList.current, t => t.id).includes(row.id)) {
        editItemList.current = _.reduce(editItemList.current, (pre: any, cur: any) => {
          if (cur.id === row.id) {
            return [...pre, {
              id: row.id, items: _.map(cur.items, h => h.salaryItemId).includes(salaryItemId) ?
                _.map(cur.items, k => {
                  if (k.salaryItemId === salaryItemId) {
                    return { ...k, salaryItemId, resultValue: row[salaryItemId] };
                  }
                  return { ...k };
                })
                : [...cur.items, { salaryItemId, resultValue: row[salaryItemId] }]
            }];
          }
          return [...pre, cur];
        }, []);
      } else {
        editItemList.current.push({
          id: row.id, items: [{ salaryItemId, resultValue: row[salaryItemId] }]
        });
      }
      window.parent.postMessage(
        {
          type: "turn",
          payload: { id: "BEATCHUPDATE", params: { resultValueList: JSON.stringify(editItemList.current) } }
        },
        "*"
      );
      newData.splice(index, 1, { ...item, ...row });
      return newData;
    });
  };

  const handleChange = (rowIndex: any, dataIndex: any, v: any, dataSource: any[]) => {
    setDataSource(() => {
      return _.map([...dataSource], (item, index) => {
        if (index === rowIndex) item[dataIndex] = v;
        return item;
      });
    });
  };
  const handleFormulaTd = (dataIndex: string) => {
    window.parent.postMessage(
      {
        type: "turn",
        payload: { id: "FORMULA", params: { dataIndex } }
      },
      "*"
    );
  };
  const handleEdit = (id: string) => {
    window.parent.postMessage(
      {
        type: "turn",
        payload: { id: "EDIT", params: { id } }
      },
      "*"
    );
  };
  const sizeChange = (pageobj: IPage) => {
  };
  const onChange = (pageobj: IPage) => {
    setPageInfo(() => {
      window.parent.postMessage(
        {
          type: "turn",
          payload: { id: "PAGEINFO", params: { ...pageInfo, ...pageobj } }
        },
        "*"
      );
      return { ...pageInfo, ...pageobj };
    });
  };
  const rowSelection = {
    columnWidth: 60, columnTitle: (isDetailTable && !showTotalCell) ? "序号" : "",
    renderCell: (value: boolean, record: any, index: number, originNode: React.ReactNode) => ((isDetailTable && !showTotalCell) ?
      <span>{index + 1}</span> : originNode),
    selectedRowKeys: selectedRowKeys,
    onChange: (selectedRowKeys: React.Key[]) => {
      setSelectedRowKeys(selectedRowKeys);
      window.parent.postMessage(
        {
          type: "turn",
          payload: { id: "CHECKBOX", params: { selectedRowKeys } }
        },
        "*"
      );
    }
  };

  const components = {
    body: {
      row: EditableRow,
      cell: EditableCell
    }
  };
  return (<Form form={form} component={false}><Table
    rowKey="id" size="small" bordered className={styles.tableWrapper}
    dataSource={dataSource} rowSelection={rowSelection}
    scroll={{ x: 1200, y: `calc(100vh - ${tableScrollHeight || (!showTotalCell ? 165 : 200)}px)` }}
    columns={!isDetailTable ? columns : _.filter(columns, o => o.dataIndex !== "operate")}
    footer={() => !isDetailTable ? <CalcExplainFooter i18n={i18n}/> : null} components={components}
    pagination={!_.isNil(pageInfo) ? {
      ...paginationFun(pageInfo, sizeChange, onChange, i18n),
      size: "default"
    } : false}
    summary={() => (
      !showTotalCell ? <></> :
        <Table.Summary fixed={fixed}>
          <Table.Summary.Row>
            <Table.Summary.Cell index={0} align="center"><Text type="danger">{i18n["总计"]}</Text></Table.Summary.Cell>
            <CaclFixedTotal columns={columns} dataSourceUrl={sumRowlistUrl} payload={payload} sumRow={sumRow}/>
          </Table.Summary.Row>
        </Table.Summary>
    )}
  /></Form>);
};

export default Index;

const EditableRow: React.FC<EditableRowProps> = (props) => {
  const [form] = Form.useForm();
  return (
    <Form form={form} component={false}>
      <EditableContext.Provider value={form}>
        <tr {...props} />
      </EditableContext.Provider>
    </Form>
  );
};
const EditableCell: React.FC<EditableCellProps> = (props) => {
  const { title, editable, children, dataIndex, record, handleSave, rowIndex, ...restProps } = props;
  const [editing, setEditing] = useState(false);
  const inputRef = useRef<any>(null);
  const form = useContext(EditableContext)!;
  useEffect(() => {
    if (editing) {
      inputRef.current!.focus();
    }
  }, [editing]);
  const toggleEdit = () => {
    setEditing(!editing);
    form.setFieldsValue({ [dataIndex]: record[dataIndex] });
  };
  const save = _.debounce(async () => {
    try {
      let values = await form.validateFields();
      if (record[`${_.keys(values)[0]}_type`] === "number") {
        values = {
          ...values,
          [_.keys(values)[0]]: !_.isNil(_.get(values, _.keys(values)[0])) ? toDecimal_n(_.get(values, _.keys(values)[0]), restProps?.pattern) : toDecimal_n(0, restProps?.pattern)
        };
      }
      handleSave?.(_.keys(values)[0], { ...record, ...values });
    } catch (errInfo) {
      console.log("Save failed:", errInfo);
    }
  }, 200);
  let childNode = children;
  if (editable) {
    childNode = editing ? (
      <Form.Item style={{ margin: 0 }} name={dataIndex}>
        {record[`${dataIndex}_type`] === "number" ?
          <InputNumber ref={inputRef} precision={restProps?.pattern} onChange={save} onBlur={toggleEdit}
                       onPressEnter={toggleEdit}/> :
          <Input ref={inputRef} onPressEnter={toggleEdit} onBlur={toggleEdit} onChange={save}/>}
      </Form.Item>
    ) : (
      // record[dataIndex]
      <div className={cs({ [styles["editable-cell"]]: true })} onClick={toggleEdit}> {children} </div>
    );
  }
  return <td {...restProps}>{childNode}</td>;
};
